# Gaaah! Messy code! You should go away for your own sake. :)

import sys
import random 
import pygame
from pygame.locals import *

import data
import util
from locals import *

def main():
    pygame.init()

    screen = pygame.display.set_mode((800, 600))
    pygame.display.set_caption('Welcome to WC')

    Menu(screen).loop()


class Menu:

    def __init__(self, screen):
        self.bg = pygame.image.load(data.filepath(IMENU)).convert()
        self.screen = screen

    def loop(self):
        self.screen.blit(self.bg, (0, 0))
        pygame.display.update()
        
        clock = pygame.time.Clock()

        done = False

        clickables = pygame.sprite.RenderUpdates()
        font_color = (0, 0, 0)
        font_scolor = (249, 236, 129)
        font = pygame.font.Font(data.filepath(MAINFONT), 60)
        text = font.render('Start Game', 1, font_color)
        stext = font.render('Start Game', 1, font_scolor)
        clickables.add(Clickable(text, stext, (self.start,), [300, 280]))
        text = font.render('Quit Game', 1, font_color)
        stext = font.render('Quit Game', 1, font_scolor)
        clickables.add(Clickable(text, stext, (self.quit,), [300, 380]))

        while not done:
            for event in pygame.event.get():
                if event.type == QUIT:
                    self.quit()
                elif event.type == KEYDOWN:
                    if event.key == K_ESCAPE:
                        self.quit()
                    elif event.key == K_SPACE:
                        self.start()
                elif event.type == MOUSEBUTTONDOWN:
                    for clickable in clickables:
                        if clickable.clicked(): break                        

            clickables.clear(self.screen, self.bg)
            clickables.update()
            pygame.display.update(clickables.draw(self.screen))
            clock.tick(MAXFPS)

    def start(self):
        Game(self.screen).loop()
        done = True

    def quit(self):
        sys.exit()


class Game:

    def __init__(self, screen):
        self.bg = pygame.Surface(screen.get_size())

        self.screen = screen

        self.message = Message([419, 384])
        
        self.player = Player(self.message)

        self.week = 0
        
        self.company = Company(self.player)
        self.actions = Actions(self)

        self.rscreen = RightScreen(self.company.employees, self.actions,
                                   self.message, self.player)
        self.room = Room(self.rscreen)
        self.sbar = StatusBar(self.player, self.week, self.actions,
                              self.rscreen)
        
        self.sprites = pygame.sprite.OrderedUpdates()
        self.sprites.add(self.room)
        self.sprites.add(self.sbar)
        self.sprites.add(self.rscreen)
        self.sprites.add(self.message)

        self.clock = pygame.time.Clock()

    def loop(self):
        done = False

        while not done:
            for event in pygame.event.get():
                if event.type == QUIT:
                    sys.exit()
                elif event.type == KEYDOWN:
                    if event.key == K_ESCAPE:
                        sys.exit()
                    elif event.key == K_SPACE:
                        self.actions.pass_turn()
                elif event.type == MOUSEBUTTONDOWN:
                    for clickable in (self.room.clickables.sprites() +
                                      self.sbar.clickables.sprites() +
                                      self.rscreen.clickables.sprites()):
                        if clickable.clicked(): break

            self.sprites.clear(self.screen, self.bg)
            self.room.clickables.clear(self.screen, self.bg)
            self.sbar.clickables.clear(self.screen, self.bg)
            self.rscreen.clickables.clear(self.screen, self.bg)
            
            self.sbar.update_text(self.week)
            self.room.clickables.update()
            self.sbar.clickables.update()
            self.rscreen.clickables.update()
            self.company.update_player_reputation()
            
            update_rects = []
            update_rects += self.sprites.draw(self.screen)
            update_rects += self.room.clickables.draw(self.screen)
            update_rects += self.sbar.clickables.draw(self.screen)
            update_rects += self.rscreen.clickables.draw(self.screen)
            pygame.display.update(update_rects)
            
            self.clock.tick(MAXFPS)

    def lose(self):
        lose = pygame.image.load(data.filepath(ILOSE)).convert()
        self.screen.blit(lose, [0, 0])
        pygame.display.update()
        self.end()

    def win(self):
        win = pygame.image.load(data.filepath(IWIN)).convert()
        self.screen.blit(win, [0, 0])
        pygame.display.update()
        self.end()        

    def end(self):
         while True:
            for event in pygame.event.get():
                if event.type == QUIT:
                    sys.exit()
                elif event.type == KEYDOWN:
                    if event.key == K_ESCAPE:
                        sys.exit()
            self.clock.tick(MAXFPS)       


class Actions:

    def __init__(self, game):
        self.game = game
        self.single_actions = SingleActions(self.game.player,
                                            self.game.message)
        self.multi_actions = MultiActions(self.game.player, self.game.message)

    def pass_turn(self):
        self.game.message.write('A week has passed...')
        self.game.week += 1
        self.game.player.pass_week()
        if self.game.week % 4 == 0:
            self.game.player.gain_salary()
        if self.game.week % 8 == 0:
            list = self.game.company.its_firing_time()
            self.game.rscreen.show_promotions(list)
            if self.game.player.rank.number == -1:
                self.game.lose()
            elif self.game.player.rank.number == 7:
                self.game.win()
        self.game.rscreen.update_screen()

    def do(self, action, parameters, target = None):
        if self.game.player.actions_left > 0:
            self.game.player.actions_left -= 1
            if action == WORK_OVERTIME:
                self.single_actions.work_overtime()
            elif action == PRACTICE:
                self.single_actions.practice(*parameters)
            elif action == PRETEND_WORK:
                self.single_actions.pretend_work()
            elif action == SOCIALIZE:
                self.multi_actions.socialize(target)
            elif action == BRIBE:
                self.multi_actions.bribe(target)
            elif action == BLACKMAIL_M:
                self.multi_actions.blackmail(target, *parameters)
            elif action == BLACKMAIL_R:
                self.multi_actions.blackmail(target, *parameters)
            elif action == EXPOSE_SECRET:
                self.multi_actions.expose_secret(target)
            elif action == SABOTAGE_WEAK:
                self.multi_actions.sabotage(target, *parameters)
            elif action == SABOTAGE_SER:
                self.multi_actions.sabotage(target, *parameters)
            elif action == SPY:
                self.multi_actions.spy(target)                
            self.game.rscreen.update_screen() 
        else:
            self.game.message.write('No actions left for this week.')

    def buy_secret(self, employee, price):
        if self.game.player.check_charge(price):
            if employee.buy_secret():
                self.game.player.charge(price)
                self.game.message.write('Secret bought.')
            else:
                self.game.message.write('No more secrets available.')
        else:
            self.game.message.write('You don\'t have enough money.')
        self.game.rscreen.update_screen()            


class Employee:

    def __init__(self):
        self.reputation = 0
        self.rank = Rank(0)
        self.name = 'John Doe'
        self.gender = 'M'

    def promote(self):
        if self.rank.number < 7:
            self.rank = Rank(self.rank.number + 1)


class Rank:

    def __init__(self, rank):
        self.number = rank
        self.name = self.get_name()
        self.salary = self.get_salary()

    def get_name(self):
        if self.number == 0:
            return 'Trainee'
        elif self.number == 1:
            return 'Worker'
        elif self.number == 2:
            return 'Supervisor'
        elif self.number == 3:
            return 'Manager'
        elif self.number == 4:
            return 'Director'
        elif self.number == 5:
            return 'Vice-President'
        elif self.number == 6:
            return 'Executive Vice-President'
        elif self.number == 7:
            return 'CEO'
        elif self.number == -1:
            return 'Fired!'
        else:
            return 'Undefined'

    def get_salary(self):
        if self.number == 0:
            return 1000
        elif self.number == 1:
            return 2500
        elif self.number == 2:
            return 4000
        elif self.number == 3:
            return 6000
        elif self.number == 4:
            return 8000
        elif self.number == 5:
            return 11000
        elif self.number == 6:
            return 14000
        elif self.number == 7:
            return 20000
        else:
            return 0
        

class Player(Employee):

    def __init__(self, message):
        Employee.__init__(self)
        self.items = []
        self.money = STARTING_MONEY
        
        self.relation = 'I love myself :)'
        self.rank = Rank(0)
        
        self.productivity = 1
        self.sociability = 1
        self.subterfuge = 1
        
        self.productivity_frac = 0
        self.sociability_frac = 0
        self.subterfuge_frac = 0
        
        self.pay = self.rank.salary
        self.pay_extra = 0
        
        self.reputation = 10
        
        self.actions_left = 5
        
        self.done_pretend = False
        
        self.message = message

    def gain_salary(self):
        paycheck = self.pay + self.pay_extra
        paycheck += (self.productivity - 1) * (paycheck / 10)
        self.message.write('You have received your paycheck: $ ' +
                           str(paycheck))
        self.money += paycheck
        self.pay_extra = 0

    def promote(self):
        Employee.promote(self)
        self.pay = self.rank.salary

    def demote(self):
        self.rank = Rank(self.rank.number - 1)
        self.pay = self.rank.salary

    def check_charge(self, price):
        return self.money >= price

    def charge(self, price):
        if self.check_charge(price):
            self.money -= price
            return True
        else:
            self.message.write('You don\'t have enough money.')
            return False

    def pass_week(self):
        self.actions_left = 5
        self.done_pretend = False

    def faked_work(self):
        if random.randint(1, 100) <= 35 + self.subterfuge * 15:
            return True
        return False


class NonPlayer(Employee):

    def __init__(self):
        Employee.__init__(self)
        self.relation = 0
        self.secrets = 0
        self.secrets_known = 0
        self.honesty = 0

    def like(self, amount):
        self.relation += amount
        if self.relation > 100:
            self.relation = 100

    def dislike(self, amount):
        self.relation -= amount
        if self.relation < 0:
            self.relation = 0

    def promote(self):
        Employee.promote(self)
        self.reputation += 10

    def buy_secret(self):
        if self.secrets == 0:
            return False
        self.secrets -= 1
        self.secrets_known += 1
        return True

    def disgrace(self, value):
        self.reputation -= value
        if self.reputation < 0:
            self.reputation = 0


class SingleActions:

    def __init__(self, player, message):
        self.player = player
        self.message = message

    def work_overtime(self):
        self.player.pay_extra += 0.05 * self.player.pay

    def practice(self, type):
        if type == PRODUCTIVITY:
            if self.player.charge(200 * self.player.productivity):
                if self.player.productivity_frac >= 9:
                    self.player.productivity_frac = 0
                    self.player.productivity += 1
                else:
                    self.player.productivity_frac += 1
            else: self.player.actions_left += 1
        elif type == SOCIABILITY:
            if self.player.charge(200 * self.player.sociability):
                if self.player.sociability_frac >= 9:
                    self.player.sociability_frac = 0
                    self.player.sociability += 1
                else:
                    self.player.sociability_frac += 1
            else: self.player.actions_left += 1                   
        elif type == SUBTERFUGE:
            if self.player.charge(200 * self.player.subterfuge):
                if self.player.subterfuge_frac >= 9:
                    self.player.subterfuge_frac = 0
                    self.player.subterfuge += 1
                else:
                    self.player.subterfuge_frac += 1
            else: self.player.actions_left += 1                    

    def pretend_work(self):
        self.player.done_pretend = True
        if self.player.faked_work():
            self.player.actions_left += 2
        else:
            self.message.write('You have been caught! Your pay was cut.')
            self.player.pay_extra -= 0.10 * self.player.pay
            if self.player.pay_extra + self.player.pay < 0:
                self.player.pay_extra = - self.player.pay


class MultiActions:

    def __init__(self, player, message):
        self.player = player
        self.message = message

    def socialize(self, target):
        if random.randint(1, 100) <= 80 + self.player.sociability * 10:
            self.message.write('The conversation was nice.')
            if target.relation < 80:
                target.like(2 * self.player.sociability)
            else:
                target.like(1 * self.player.sociability)
        else:
            self.message.write('You two ended up in a tense mood.')
            target.dislike(2 / self.player.sociability)

    def bribe(self, target):
        if self.player.check_charge((target.rank.number + 1) * 1000):
            if target.honesty < 30 + self.player.subterfuge * 10:
                self.message.write('The bribe was accepted.')
                self.player.charge((target.rank.number + 1) * 1000)
                target.like(30)
            else:
                self.message.write('The bribe was rejected.')
                target.dislike(15)
        else:
            self.message.write('You don\'t have enough money to bribe.')

    def blackmail(self, target, type):
        if target.secrets_known > 0:
            if random.randint(1, 100) <= (100 - target.honesty +
                                          self.player.subterfuge * 10):
                target.secrets_known -= 1
                if type == 1:
                    paid = 600 + 300 * target.rank.number * target.rank.number
                    self.player.money += paid
                    self.message.write('Your target has paid you $' +
                                       str(paid) + ' to stay silent.')
                else:
                    target.like(50)
                    self.message.write('Your target will talk good about you' +
                                       ' so you stay silent.')
            else:
                self.message.write('Your target has refused to pay you' +
                                   ' blackmail.')
                target.dislike(20)
        else:
            self.message.write('You have no secrets to use as blackmail.')
            self.player.actions_left += 1

    def expose_secret(self, target):
        if target.secrets_known > 0:
            target.secrets_known -= 1
            target.disgrace(30)
            target.dislike(40)
            self.message.write('You expose the secret, ' +
                               'and people are horrified!')
        else:
            self.message.write('You have no secrets to expose.')
            self.player.actions_left += 1

    def sabotage(self, target, intensity):
        if random.randint(1, 100) <= (self.player.subterfuge * 10 +
                                      target.honesty * 5):
            target.disgrace(intensity * 10)
            if intensity == 1:
                self.message.write('You successfully made your target ' +
                                   'look like a fool.')
            else:
                self.message.write('You successfully blamed your ' +
                                   'target for the destruction.')
        else:
            target.dislike(intensity * 10)
            self.message.write('You were caught trying to sabotage ' +
                               'your target.')

    def spy(self, target):
        if random.randint(1, 100) <= (target.secrets * 10 +
                                      self.player.subterfuge * 10):
            if target.buy_secret():
                self.message.write('You found out a secret!')
            else:
                self.message.write('You discovered there are no secrets left.')
        else:
            target.dislike(10)
            self.message.write('You were caught snooping around.')


class Company:

    def __init__(self, player):
        self.ranks = [3, 16, 8, 6, 4, 3, 1, 1]
        self.player = player
        self.employees = [self.player] + (EmployeeGenerator().
                                          generate_office(self.ranks))

    def all_reputation(self):
        rep = [0, 0, 0, 0, 0, 0, 0, 0]
        for employee in self.employees:
            if employee is not self.player:
                rep[employee.rank.number] += employee.relation
        for i in range(8):
            rep[i] /= self.ranks[i]
        return rep

    def level_reputation(self, rep, rank):
        reputation = (rep[max(rank - 1, 0)] * 0.10 +
                      rep[rank] * 0.20 +
                      rep[min(rank + 1, 7)] * 0.30 +
                      rep[min(rank + 2, 7)] * 0.40)
        return reputation

    def update_player_reputation(self):
        rep = self.all_reputation()
        reputation = int(self.level_reputation(rep, self.player.rank.number))
        self.player.reputation = reputation

    def choose_fired(self):
        low_employee = self.employees[0]
        for employee in self.employees:
            if employee.rank.number >= self.player.rank.number - 1:
                if employee.reputation < low_employee.reputation:
                    low_employee = employee
        return low_employee

    def choose_promoted(self, rank):
        max_rep = -1
        for employee in self.employees:
            if employee.rank.number == rank and employee.reputation > max_rep:
                promoted = employee
                max_rep = employee.reputation
        return promoted

    def its_firing_time(self):
        list = []
        fired = self.choose_fired()
        if fired == self.player:
            if self.player.rank.number != 0:
                promoted = self.choose_promoted(self.player.rank.number - 1)
                self.promote(promoted)
            self.player.demote()
            if self.player.rank.number == -1:
                list.append(self.player.name + ' fired!')
            else:
                list.append(self.player.name + ' demoted!')
                list.append(promoted.name + ' promoted!')
        else:
            list.append(fired.name + ' fired!')
            for i in range(fired.rank.number):
                promoted = self.choose_promoted(i)
                self.promote(promoted)
                list.append(promoted.name + ' promoted!')
            trainee = self.hire_trainee()
            list.append(trainee.name + ' hired!')
            self.fire(fired)
        return list

    def fire(self, employee):
        self.employees.remove(employee)

    def promote(self, employee):
        employee.promote()

    def hire_trainee(self):
        trainee = EmployeeGenerator().generate_one(0)
        self.employees.append(trainee)
        return trainee
        

class EmployeeGenerator:

    def __init__(self):
        self.male_names = self.load_names(data.filepath(DMALE))
        self.female_names = self.load_names(data.filepath(DFEMALE))
        self.last_names = self.load_names(data.filepath(DLAST))

    def load_names(self, file):
        f = open(file, 'r')
        list = []
        line = f.readline().strip()
        while line != '':
            list.append(line)
            line = f.readline().strip()
        return list

    def generate_one(self, rank):
        employee = NonPlayer()
        employee.gender = random.choice(['M', 'F'])
        if employee.gender == 'M':
            employee.name = random.choice(self.male_names)
        else:
            employee.name = random.choice(self.female_names)
        employee.name += ' ' + random.choice(self.last_names)

        employee.rank = Rank(rank)
        if employee.rank.number == 7:
            employee.reputation = 95
        else:
            employee.reputation = random.randint(10 + employee.rank.number*10,
                                                 20 + employee.rank.number*10)
        employee.relation = random.randint(10, 20)
        employee.honesty = random.randint(0, 100)

        employee.secrets = 10 - employee.honesty / 10 + random.randint(-2, 2)
        if employee.secrets < 0: employee.secrets = 0

        return employee

    def generate_office(self, ranks):
        employees = []
        for i in range(8):
            for j in range(ranks[i]):
                employees.append(self.generate_one(i))
        return employees

        
class StatusBar(pygame.sprite.Sprite):

    def __init__(self, player, week, actions, rscreen):
        pygame.sprite.Sprite.__init__(self)

        self.bg = pygame.image.load(data.filepath(ISTATUSBAR)).convert()
        self.image = pygame.Surface((800, 150))
        self.rect = self.image.get_rect()
        self.rect.bottomleft = [0, 600]

        self.guy = pygame.image.load(data.filepath(IGUY)).convert()
        self.sguy = pygame.image.load(data.filepath(ISGUY)).convert()

        self.player = player

        self.fontl = pygame.font.Font(data.filepath(MAINFONT), 32)
        self.fontm = pygame.font.Font(data.filepath(MAINFONT), 26)
        self.font_color = (36, 36, 180)

        self.months = ['January', 'February', 'March', 'April', 'May',
                       'June', 'July', 'August', 'September', 'October',
                       'November', 'December']
        self.starting_year = 2007
        
        self.im_snext_week = (pygame.image.load(data.filepath(ISNEXTWEEK)).
                             convert())
        self.im_next_week = pygame.Surface(self.im_snext_week.get_size())
        self.im_next_week.set_colorkey((0, 0, 0))

        self.clickables = pygame.sprite.RenderUpdates()

        self.rscreen = rscreen
        self.actions = actions

        self.update_text(week)           

    def update_text(self, week):
        
        self.image.blit(self.bg, (0, 0))
        
        self.clickables = pygame.sprite.RenderUpdates()
        self.clickables.add(Clickable(self.im_next_week, self.im_snext_week,
                                      (self.actions.pass_turn,),
                                      [740, 550]))      
        self.clickables.add(Clickable(self.guy, self.sguy,
                                      (self.rscreen.show_single,),
                                      [0, 450]))

        text = self.fontl.render(self.player.name, 1, self.font_color)
        self.image.blit(text, (100, 0))
        text = self.fontl.render(self.player.rank.name, 1, self.font_color)
        self.image.blit(text, (100, 23))

        text = self.fontm.render('$ ' + str(self.player.money), 1,
                                 self.font_color)
        self.image.blit(text, (100, 55))

        text = self.fontm.render('Productivity', 1, self.font_color)
        self.image.blit(text, (250, 4))
        text = self.fontl.render('x ' * self.player.productivity, 1,
                                  self.font_color)
        self.image.blit(text, (250, 18))
        
        text = self.fontm.render('Sociability', 1, self.font_color)
        self.image.blit(text, (250, 54))
        text = self.fontl.render('x ' * self.player.sociability, 1,
                                  self.font_color)
        self.image.blit(text, (250, 68))
        
        text = self.fontm.render('Subterfuge', 1, self.font_color)
        self.image.blit(text, (250, 94))
        text = self.fontl.render('x ' * self.player.subterfuge, 1,
                                  self.font_color)
        self.image.blit(text, (250, 108))

        text = self.fontm.render('Reputation', 1, self.font_color)
        self.image.blit(text, (400, 5))

        text = self.fontm.render(util.bar(self.player.reputation) +
                                 ' ' + str(self.player.reputation) + '%',
                                 1, self.font_color)
        self.image.blit(text, (400, 35))
                           
        text = self.fontm.render(self.get_date(week), 1, self.font_color)
        self.image.blit(text, (400, 115))

        text = self.fontm.render(str(self.player.actions_left) +
                                 ' actions left', 1, self.font_color)
        self.image.blit(text, (670, 5))

    def get_date(self, week):
        return ('Week ' + str(week % 4 + 1) + ', ' +
                self.months[(week / 4) % 12] + ', ' +
                str(self.starting_year + (week / 48)))
                

class Room(pygame.sprite.Sprite):

    def __init__(self, rscreen):
        pygame.sprite.Sprite.__init__(self)

        self.bg_supply = pygame.image.load(data.filepath(ISUPPLY)).convert()
        self.bg_office = pygame.image.load(data.filepath(IOFFICE)).convert()
        self.bg_hoffice = pygame.image.load(data.filepath(IHOFFICE)).convert()
        self.bg_voffice = pygame.image.load(data.filepath(IVOFFICE)).convert()
        self.bg_moffice = pygame.image.load(data.filepath(IMOFFICE)).convert()
        
        self.image = pygame.Surface((800, 450))
        self.rect = self.image.get_rect()
        self.rect.topleft = [0, 0]

        self.fontl = pygame.font.Font(data.filepath(MAINFONT), 40)
        self.fontm = pygame.font.Font(data.filepath(MAINFONT), 32)
        self.fonts = pygame.font.Font(data.filepath(MAINFONT), 26)
        
        self.font_color = (255, 255, 255)
        self.font_scolor = (249, 236, 129)

        self.clickables = pygame.sprite.RenderUpdates()
        self.im_sup = pygame.image.load(data.filepath(ISUP)).convert()
        self.im_up = pygame.Surface(self.im_sup.get_size())
        self.im_up.set_colorkey((0, 0, 0))
        self.im_sdown = pygame.image.load(data.filepath(ISDOWN)).convert()
        self.im_down = pygame.Surface(self.im_sdown.get_size())
        self.im_down.set_colorkey((0, 0, 0))

        self.rscreen = rscreen

        self.move(1)

    def move(self, room):
        self.room = room
        if room == 0:
            self.supply_room()
        elif room == 1:
            self.office()
        elif room == 2:
            self.high_office()
        elif room == 3:
            self.very_high_office()
        else:
            self.master_office()
        
    def supply_room(self):

        self.image.blit(self.bg_supply, (0, 0))
        self.clickables = pygame.sprite.RenderUpdates()      

        text = self.fontl.render('Supply Room', 1, self.font_color)
        self.image.blit(text, (10, 370))

        self.clickables.add(Clickable(self.im_up, self.im_sup,
                                      (self.move, 1), [359, 150]))

        text = self.fontm.render('Dark Corner', 1, self.font_color)
        self.image.blit(text, (79, 26))

        text = self.fontm.render('The Janitor', 1, self.font_color)
        stext = self.fontm.render('The Janitor', 1, self.font_scolor)
        self.clickables.add(Clickable(text, stext,
                                      (self.rscreen.show_shop,),
                                      [239, 257]))

        text = self.fontm.render('Door', 1, self.font_color)
        self.image.blit(text, (90, 230))

    def office(self):

        self.image.blit(self.bg_office, (0, 0))
        self.clickables = pygame.sprite.RenderUpdates()

        text = self.fontl.render('Office', 1, self.font_color)
        self.image.blit(text, (10, 370))

        text = self.fonts.render('Elevator', 1, self.font_color)
        self.image.blit(text, (340, 114))

        self.clickables.add(Clickable(self.im_up, self.im_sup,
                                      (self.move, 2), [359, 150]))
        self.clickables.add(Clickable(self.im_down, self.im_sdown,
                                      (self.move, 0), [359, 181]))        

        text = self.fontm.render('Trainees', 1, self.font_color)
        stext = self.fontm.render('Trainees', 1, self.font_scolor)
        self.clickables.add(Clickable(text, stext,
                                      (self.rscreen.show_employees, Rank(0)),
                                      [302, 20]))

        text = self.fontm.render('Workers', 1, self.font_color)
        stext = self.fontm.render('Workers', 1, self.font_scolor)
        self.clickables.add(Clickable(text, stext,
                                      (self.rscreen.show_employees, Rank(1)),
                                      [198, 20]))

        text = self.fontm.render('Supervisors', 1, self.font_color)
        stext = self.fontm.render('Supervisors', 1, self.font_scolor)
        self.clickables.add(Clickable(text, stext,
                                      (self.rscreen.show_employees, Rank(2)),
                                      [55, 37]))

        text = self.fontm.render('Mmm... Coffee', 1, self.font_color)
        self.image.blit(text, (36, 267))

    def high_office(self):

        self.image.blit(self.bg_hoffice, (0, 0))
        self.clickables = pygame.sprite.RenderUpdates()
        
        text = self.fontl.render('High Office', 1, self.font_color)
        self.image.blit(text, (10, 370))

        text = self.fonts.render('Elevator', 1, self.font_color)
        self.image.blit(text, (340, 114))

        self.clickables.add(Clickable(self.im_up, self.im_sup,
                                      (self.move, 3), [359, 150]))
        self.clickables.add(Clickable(self.im_down, self.im_sdown,
                                      (self.move, 1), [359, 181]))
        
        text = self.fontm.render('Managers', 1, self.font_color)
        stext = self.fontm.render('Managers', 1, self.font_scolor)
        self.clickables.add(Clickable(text, stext,
                                      (self.rscreen.show_employees, Rank(3)),
                                      [240, 44]))

        text = self.fontm.render('Directors', 1, self.font_color)
        stext = self.fontm.render('Directors', 1, self.font_scolor)
        self.clickables.add(Clickable(text, stext,
                                      (self.rscreen.show_employees, Rank(4)),
                                      [62, 40]))

    def very_high_office(self):

        self.image.blit(self.bg_voffice, (0, 0))
        self.clickables = pygame.sprite.RenderUpdates()
        
        text = self.fontl.render('Very High Office', 1, self.font_color)
        self.image.blit(text, (10, 370))

        text = self.fonts.render('Elevator', 1, self.font_color)
        self.image.blit(text, (340, 114))

        self.clickables.add(Clickable(self.im_up, self.im_sup,
                                      (self.move, 4), [359, 150]))
        self.clickables.add(Clickable(self.im_down, self.im_sdown,
                                      (self.move, 2), [359, 181]))
        
        text = self.fontm.render('Vice-Presidents', 1, self.font_color)
        stext = self.fontm.render('Vice-Presidents', 1, self.font_scolor)
        self.clickables.add(Clickable(text, stext,
                                      (self.rscreen.show_employees, Rank(5)),
                                      [242, 41]))

        text = self.fontm.render('Executive VP', 1, self.font_color)
        stext = self.fontm.render('Executive VP', 1, self.font_scolor)
        self.clickables.add(Clickable(text, stext,
                                      (self.rscreen.show_employees, Rank(6)),
                                      [94, 38]))

    def master_office(self):

        self.image.blit(self.bg_moffice, (0, 0))
        self.clickables = pygame.sprite.RenderUpdates()
        
        text = self.fontl.render('Master Office', 1, self.font_color)
        self.image.blit(text, (10, 370))

        text = self.fonts.render('Elevator', 1, self.font_color)
        self.image.blit(text, (340, 114))

        self.clickables.add(Clickable(self.im_down, self.im_sdown,
                                      (self.move, 3), [359, 181]))

        text = self.fontm.render('CEO', 1, self.font_color)
        stext = self.fontm.render('CEO', 1, self.font_scolor)
        self.clickables.add(Clickable(text, stext,
                                      (self.rscreen.show_employees, Rank(7)),
                                      [135, 23]))


class RightScreen(pygame.sprite.Sprite):

    def __init__(self, employees, actions, message, player):
        pygame.sprite.Sprite.__init__(self)

        self.bg = pygame.image.load(data.filepath(IRIGHTBG)).convert()
        self.image = pygame.Surface((400, 450))
        self.rect = self.image.get_rect()
        self.rect.topleft = [400, 0]

        self.fontl = pygame.font.Font(data.filepath(MAINFONT), 40)
        self.fontm = pygame.font.Font(data.filepath(MAINFONT), 32)
        self.fonts = pygame.font.Font(data.filepath(MAINFONT), 26)

        self.font_color = (255, 255, 255)
        self.font_scolor = (249, 236, 129)
        self.font_gcolor = (200, 200, 200)

        self.employees = employees

        self.clickables = pygame.sprite.RenderUpdates()

        self.actions = actions
        self.message = message
        self.player = player

        self.empty()

    def update_screen(self):
        self.last[0](*self.last[1:])

    def empty(self):
        self.clickables = pygame.sprite.RenderUpdates()
        self.image.blit(self.bg, (0, 0))
        self.last = (self.empty,)

    def show_employees(self, rank, secrets = False, price = 0):
        self.empty()
        self.last = (self.show_employees, rank, secrets, price)
        voffset = 80
        hoffset = 420

        if secrets == False:
            plural = ''
            if rank.number < 6: plural = 's'
            text = self.fontl.render(rank.name + plural, 1, self.font_color)
        else:
            text = self.fontl.render('The Janitor\'s Secret Shop of Secrets',
                                     1, self.font_color)
        
        self.image.blit(text, (20, 20))
        
        for employee in self.employees:
            if employee.rank.number == rank.number:
                if secrets == False:
                    text = self.fonts.render(employee.name, 1, self.font_color)
                    stext = self.fonts.render(employee.name, 1,
                                              self.font_scolor)
                    if employee == self.player:
                        msg = ('Relation: ' + employee.relation + '         ' +
                               'Reputation: ' + util.bar(employee.reputation) +
                               ' ' + str(employee.reputation) + '%')
                    else:
                        msg = ('Relation: ' + util.bar(employee.relation) +
                               ' ' + str(employee.relation) + '%        ' +
                               'Reputation: ' + util.bar(employee.reputation) +
                               ' ' + str(employee.reputation) + '%')
                    if employee == self.player:
                        function = (self.show_single,)
                    else:
                        function = (self.show_employee, employee)
                    emp_click = Clickable(text, stext, function,
                                          [hoffset, voffset],
                                          (self.message.small_write, msg))
                else:
                    if employee == self.player: continue
                    txt = (employee.name + ' ......... ' +
                           str(employee.secrets) + ' available')
                    text = self.fonts.render(txt, 1, self.font_color)
                    stext = self.fonts.render(txt, 1, self.font_scolor)
                    emp_click = Clickable(text, stext,
                                          (self.actions.buy_secret, employee,
                                           price),
                                          [hoffset, voffset])
                voffset += 36
                if voffset >= 350:
                    voffset = 80
                    hoffset = 620
                self.clickables.add(emp_click)

    def show_employee(self, employee):
        self.empty()
        self.last = (self.show_employee, employee)
        text = self.fontl.render(employee.name, 1, self.font_color)
        self.image.blit(text, (20, 20))
        text = self.fontl.render(employee.rank.name, 1, self.font_color)
        self.image.blit(text, (20, 60))
        if employee == self.player:
            text = self.fonts.render('Relation: ' + str(employee.relation),
                                     1, self.font_color)
        else:
            text = self.fonts.render('Relation: ' + str(employee.relation) +
                                     '%', 1, self.font_color)           
        self.image.blit(text, (20, 120))
        text = self.fonts.render('Reputation: ' + str(employee.reputation) +
                                 '%', 1,  self.font_color)
        self.image.blit(text, (20, 150))

        if employee == self.player: return

        text = self.fonts.render('Secrets available: ' +
                                 str(employee.secrets), 1,
                                 self.font_color)
        self.image.blit(text, (220, 120))

        text = self.fonts.render('Secrets known: ' +
                                 str(employee.secrets_known), 1,
                                 self.font_color)
        self.image.blit(text, (220, 150))        

        txt = 'Socialize'
        text = self.fonts.render(txt, 1, self.font_color)
        stext = self.fonts.render(txt, 1, self.font_scolor)
        self.clickables.add(Clickable(text, stext,
                                      (self.actions.do, SOCIALIZE, (),
                                       employee), [420, 200]))
        
        txt = 'Bribe $' + str((employee.rank.number + 1) * 1000)
        text = self.fonts.render(txt, 1, self.font_color)
        stext = self.fonts.render(txt, 1, self.font_scolor)
        self.clickables.add(Clickable(text, stext,
                                      (self.actions.do, BRIBE, (), employee),
                                      [420, 240]))
        
        txt = 'Blackmail for Money'
        text = self.fonts.render(txt, 1, self.font_color)
        stext = self.fonts.render(txt, 1, self.font_scolor)
        self.clickables.add(Clickable(text, stext,
                                      (self.actions.do, BLACKMAIL_M, (1,),
                                       employee), [420, 280]))
        
        txt = 'Blackmail for Reputation'
        text = self.fonts.render(txt, 1, self.font_color)
        stext = self.fonts.render(txt, 1, self.font_scolor)
        self.clickables.add(Clickable(text, stext,
                                      (self.actions.do, BLACKMAIL_R, (2,),
                                       employee), [420, 320]))
        
        txt = 'Expose Secret'
        text = self.fonts.render(txt, 1, self.font_color)
        stext = self.fonts.render(txt, 1, self.font_scolor)
        self.clickables.add(Clickable(text, stext,
                                      (self.actions.do, EXPOSE_SECRET, (),
                                       employee), [620, 200]))
        
        txt = 'Sabotage (Weak)'
        text = self.fonts.render(txt, 1, self.font_color)
        stext = self.fonts.render(txt, 1, self.font_scolor)
        self.clickables.add(Clickable(text, stext,
                                      (self.actions.do, SABOTAGE_WEAK, (1,),
                                       employee), [620, 240]))
        
        txt = 'Sabotage (Serious)'
        text = self.fonts.render(txt, 1, self.font_color)
        stext = self.fonts.render(txt, 1, self.font_scolor)
        self.clickables.add(Clickable(text, stext,
                                      (self.actions.do, SABOTAGE_SER, (2,),
                                       employee), [620, 280]))
        
        txt = 'Spy'
        text = self.fonts.render(txt, 1, self.font_color)
        stext = self.fonts.render(txt, 1, self.font_scolor)
        self.clickables.add(Clickable(text, stext,
                                      (self.actions.do, SPY, (), employee),
                                      [620, 320]))       

    def show_shop(self):
        self.empty()
        self.last = (self.show_shop,)
        text = self.fontl.render('The Janitor\'s Secret Shop of Secrets', 1,
                                 self.font_color)
        self.image.blit(text, (20, 20))

        for i in range(8):
            price = 300 + 150 * i * i
            rank = Rank(i)
            txt = rank.name + ' .......... $' + str(price)
            text = self.fonts.render(txt, 1, self.font_color)
            stext = self.fonts.render(txt, 1, self.font_scolor)
            self.clickables.add(Clickable(text, stext,
                                          (self.show_employees, rank, True,
                                           price), [420, 90 + i*36]))
            
    def show_single(self):
        self.empty()
        self.last = (self.show_single,)
        text = self.fontl.render('Single Actions', 1,
                                 self.font_color)
        self.image.blit(text, (20, 20))

        txt = ('Work Overtime (current extra pay is ' +
               str(self.player.pay_extra) +  ')')
        text = self.fonts.render(txt, 1, self.font_color)
        stext = self.fonts.render(txt, 1, self.font_scolor)
        self.clickables.add(Clickable(text, stext,
                                      (self.actions.do, WORK_OVERTIME, ()),
                                      [420, 100]))

        if self.player.productivity >= 5:
            txt = 'Practice Productivity (maxed out)'
            text = self.fonts.render(txt, 1, self.font_gcolor)
            self.image.blit(text, [20, 150])
        else:
            txt = ('Practice Productivity (' +
                   str(self.player.productivity_frac * 10) + '%): costs $' +
                   str(self.player.productivity * 200))
            text = self.fonts.render(txt, 1, self.font_color)
            stext = self.fonts.render(txt, 1, self.font_scolor)
            self.clickables.add(Clickable(text, stext,
                                          (self.actions.do, PRACTICE,
                                           (PRODUCTIVITY,)), [420, 150]))
        if self.player.sociability >= 5:
            txt = 'Practice Sociability (maxed out)'
            text = self.fonts.render(txt, 1, self.font_gcolor)
            self.image.blit(text, [20, 200])
        else:
            txt = ('Practice Sociability (' +
                   str(self.player.sociability_frac * 10) + '%): costs $' +
                   str(self.player.sociability * 200))
            text = self.fonts.render(txt, 1, self.font_color)
            stext = self.fonts.render(txt, 1, self.font_scolor)
            self.clickables.add(Clickable(text, stext,
                                          (self.actions.do, PRACTICE,
                                           (SOCIABILITY,)), [420, 200]))

        if self.player.subterfuge >= 5:
            txt = 'Practice Subterfuge (maxed out)'
            text = self.fonts.render(txt, 1, self.font_gcolor)
            self.image.blit(text, [20, 250])
        else:
            txt = ('Practice Subterfuge (' +
                   str(self.player.subterfuge_frac * 10) + '%): costs $' +
                   str(self.player.subterfuge * 200))
            text = self.fonts.render(txt, 1, self.font_color)
            stext = self.fonts.render(txt, 1, self.font_scolor)
            self.clickables.add(Clickable(text, stext,
                                      (self.actions.do, PRACTICE,
                                       (SUBTERFUGE,)), [420, 250]))

        txt = 'Pretend to Work'
        if self.player.done_pretend:
            txt += ' (done for the week)'
            text = self.fonts.render(txt, 1, self.font_gcolor)           
            self.image.blit(text, [20, 300])
        else:
            text = self.fonts.render(txt, 1, self.font_color)
            stext = self.fonts.render(txt, 1, self.font_scolor)
            self.clickables.add(Clickable(text, stext,
                                          (self.actions.do, PRETEND_WORK, ()),
                                          [420, 300]))

    def show_promotions(self, list):
        self.empty()
        self.last = (self.show_promotions, list)
        text = self.fontl.render('Promotion Month Summary', 1,
                                 self.font_color)
        self.image.blit(text, (20, 20))
        voffset = 80
        for txt in list:
            text = self.fonts.render(txt, 1, self.font_color)
            self.image.blit(text, (20, voffset))
            voffset += 40


class Clickable(pygame.sprite.Sprite):

    def __init__(self, image, image_selected, click, pos, selectf = None):
        pygame.sprite.Sprite.__init__(self)

        self.image = image
        self.image_normal = image
        self.image_selected = image_selected
        self.rect = self.image.get_rect()
        self.rect.topleft = pos
        
        self.click = click
        self.selectf = selectf

    def select(self):
        self.image = self.image_selected
        if self.selectf is not None:
            self.selectf[0](*self.selectf[1:])

    def deselect(self):
        self.image = self.image_normal

    def check(self):
        return util.inside(pygame.mouse.get_pos(), self.rect.topleft,
                           self.rect.bottomright)

    def update(self):
        if self.check():
            self.select()
        else: self.deselect()

    def clicked(self):
        if self.check():
            # Click is a list, first elem is function, the rest are parameters
            self.click[0](*self.click[1:])
            return True
        return False


class Message(pygame.sprite.Sprite):

    def __init__(self, pos):
        pygame.sprite.Sprite.__init__(self)

        self.font = pygame.font.Font(data.filepath(MAINFONT), 26)
        self.fonts = pygame.font.Font(data.filepath(MAINFONT), 22)
        self.clear()
        self.pos = pos       

    def write(self, text):
        self.image = self.font.render(text, 1, (255, 255, 255))
        self.rect = self.image.get_rect()
        self.rect.topleft = self.pos

    def clear(self):
        self.image = pygame.Surface((0, 0))
        self.rect = self.image.get_rect()

    def small_write(self, text):
        self.image = self.fonts.render(text, 1, (255, 255, 255))
        self.rect = self.image.get_rect()
        self.rect.topleft = self.pos        


if __name__ == "__main__":
    main()
